"""
File name:		pollard.py

Description:	Implementation of the Pollard rho factorization method

Author:			Eli Howey
				Written for Final Project
				MATH 56, Spring 2014

"""

import matplotlib.pyplot as plt
import time

def pollard(n):
	"""
	pollard(n)
	Finds a factor of n using the Pollard rho factorization method

	Returns
	"""
	if n <= 3:
		return "Error: pollard: n must be greater than 3"

	MAX_ITER = int(sqrt(n))
	num_iter = 0

	g = n		# The factor; initialize to n to ensure loop runs once

	# While g is a trivial factor, search for factors
	while num_iter < MAX_ITER and (g == 1 or g == n):
		num_iter += 1

		# Randomly choose seeds; finds least factor p in O(sqrt(p)) steps
		# Reselect seeds if a bad seed is used, i.e., if g = n
		if g == n:
			a = randint(1,n-3)
			s = randint(0,n-1)

			U = V = s
			F = lambda x: (x * x + a) % n

		# Search for factors (Floyd cycle-finding method)
		U, V = F(U), F(F(V))
		g = gcd(U - V, n)

	if num_iter == MAX_ITER:
		return "Error: pollard: maximum # iterations performed"

	return g



def pollard_test():
	print "\nTest of pollard\n"


	# n = 2*2*2 * 3*3 * 5*5
	# print "Randomness demonstration: n = {0} = {1}".format(n, factor(n))

	# for i in range(5):
	# 	p = pollard(n)
	# 	print "pollard (run {0}): {1} = {2}".format(i+1, p, factor(p))
	# print ""

	# n = Primes().unrank(20)		# 20th prime
	# print "What if n is prime? n = {0}".format(n)
	# print pollard(n), "\n"

	# Test with n product of two primes, roughly growing by factors of 10
	N = [7*13, 29*43, 73*139, 281*337, 743*1303]
	T = []		# Runtimes for each

	for n in N:
		t = time.time()
		pollard(n)
		T.append(time.time() - t)

	print T

	plt.semilogx(N,T)
	plt.xlabel('n')
	plt.ylabel('Runtime (s)')
	plt.show()



pollard_test()


